// -------------------------------------------------------------------------- //
// Demonstration of the method for combining Lorenz curves
// -------------------------------------------------------------------------- //

clear

// -------------------------------------------------------------------------- //
// Graphical theme
// -------------------------------------------------------------------------- //

set scheme s2color
grstyle init
grstyle color background white
grstyle anglestyle vertical_tick horizontal
grstyle yesno draw_major_hgrid yes
grstyle yesno grid_draw_min yes
grstyle yesno grid_draw_max yes
grstyle color grid                   gs13
grstyle color major_grid             gs13
grstyle color minor_grid             gs13
grstyle linewidth major_grid thin

grstyle linewidth foreground   vvthin
grstyle linewidth background   vvthin
grstyle linewidth grid         vvthin
grstyle linewidth major_grid   vvthin
grstyle linewidth minor_grid   vvthin
grstyle linewidth tick         vvthin
grstyle linewidth minortick    vvthin

grstyle yesno extend_grid_low        yes
grstyle yesno extend_grid_high       yes
grstyle yesno extend_minorgrid_low   yes
grstyle yesno extend_minorgrid_high  yes
grstyle yesno extend_majorgrid_low   yes
grstyle yesno extend_majorgrid_high  yes

grstyle clockdir legend_position     6
grstyle gsize legend_key_xsize       8
grstyle color legend_line            background
grstyle yesno legend_force_draw      yes

grstyle margin axis_title          medsmall

// -------------------------------------------------------------------------- //
// Examples of survey and tax data Lorenz curves
// -------------------------------------------------------------------------- //

// Generate g-percentiles
set obs 127
generate p = .
replace p = 1000*(_n - 1)          in 1/100
replace p = 99000 + 100*(_n - 100) in 101/109
replace p = 99900 + 10*(_n - 109)  in 110/118
replace p = 99990 + (_n - 118)     in 119/127

// p normalized within [0, 1]
generate p_std = p/1e5

// Size of each percentile
generate n = p[_n + 1] - p
replace n = 1 if p == 99999

// Generate survey Lorenz curve (functional form is just an example)
generate l_svy = ibeta(3, 1, p_std)

// Generate tax data Lorenz curve (functional form is just an example)
generate l_tax = ibeta(1, 0.5, p_std) if p >= 90000

// Compare Lorenz curves
/*
gr tw line l_svy l_tax p_std, aspectratio(1) xlabel(0(0.1)1, grid) ylabel(0(0.1)1) ///
	xtitle("cumulative population share (%)") ytitle("cumulative income share") ///
	xlabel(0 "0" 0.1 "10" 0.2 "20" 0.3 "30" 0.4 "40" 0.5 "50" 0.6 "60" 0.7 "70" 0.8 "80" 0.9 "90" 1 "100") ///
	ylabel(0 "0" 0.1 "10" 0.2 "20" 0.3 "30" 0.4 "40" 0.5 "50" 0.6 "60" 0.7 "70" 0.8 "80" 0.9 "90" 1 "100") ///
	legend(pos(3) col(1) label(1 "Survey") label(2 "Tax data"))
*/

// -------------------------------------------------------------------------- //
// Step 1: Rescaling at the bottom
// -------------------------------------------------------------------------- //

// Retrieve bottom 90% share in the tax data
levelsof l_tax if p == 90000, local(bottomshare_tax)

// Retrieve bottom 90% share in the survey data
levelsof l_svy if p == 90000, local(bottomshare_svy)

// Rescale the survey Lorenz curve
generate l_svy_resc = l_svy/`bottomshare_svy'*`bottomshare_tax'

// Combine both Lorenz curves
generate l_resc = cond(p < 90000, l_svy_resc, l_tax)

// Compare Lorenz curves
/*
gr tw line l_svy l_resc p_std, aspectratio(1) xlabel(0(0.1)1, grid) ylabel(0(0.1)1) ///
	xtitle("cumulative population share (%)") ytitle("cumulative income share") ///
	xlabel(0 "0" 0.1 "10" 0.2 "20" 0.3 "30" 0.4 "40" 0.5 "50" 0.6 "60" 0.7 "70" 0.8 "80" 0.9 "90" 1 "100") ///
	ylabel(0 "0" 0.1 "10" 0.2 "20" 0.3 "30" 0.4 "40" 0.5 "50" 0.6 "60" 0.7 "70" 0.8 "80" 0.9 "90" 1 "100") ///
	legend(pos(3) col(1) label(1 "Survey") label(2 "Combined" "(no adjustment)"))
*/
	
// Notice the issue with the implied quantile of the new distribution
generate q_tax = (cond(_n < _N, l_tax[_n + 1], 1) - l_tax)/n*1e5
generate q_svy = (cond(_n < _N, l_svy[_n + 1], 1) - l_svy)/n*1e5
generate q_resc = (cond(_n < _N, l_resc[_n + 1], 1) - l_resc)/n*1e5

/*
gr tw line q_resc p_std if p <= 98000, xtitle("population percentile (%)") ytitle("(normalized) quantile") ///
	xlabel(0 "0" 0.1 "10" 0.2 "20" 0.3 "30" 0.4 "40" 0.5 "50" 0.6 "60" 0.7 "70" 0.8 "80" 0.9 "90" 1 "100") ///
	legend(off)
*/
	
// -------------------------------------------------------------------------- //
// Correction method 1: Re-ranking
// -------------------------------------------------------------------------- //

// Store a re-ranked version of the quantiles
preserve
	keep q_resc
	sort q_resc
	rename q_resc q_sort
	tempfile q_sort
	save "`q_sort'"
restore

// Match with the initial tabulation on the row number
merge 1:1 _n using "`q_sort'", nogenerate

// Compare the quantile functions
/*
gr tw line q_resc q_sort p_std if p <= 98000, xtitle("population percentile (%)") ytitle("(normalized) quantile") ///
	xlabel(0 "0" 0.1 "10" 0.2 "20" 0.3 "30" 0.4 "40" 0.5 "50" 0.6 "60" 0.7 "70" 0.8 "80" 0.9 "90" 1 "100") ///
	legend(row(1) label(1 "Unadjusted") label(2 "Adjusted via re-ranking"))
*/
	
// New top shares
gsort -p
generate l_sort = 1 - sum(q_sort*n/1e5)
gsort p

// Compare Lorenz curves
/*
gr tw line l_resc l_sort p_std, aspectratio(1) xlabel(0(0.1)1, grid) ylabel(0(0.1)1) ///
	xtitle("cumulative population share (%)") ytitle("cumulative income share") ///
	xlabel(0 "0" 0.1 "10" 0.2 "20" 0.3 "30" 0.4 "40" 0.5 "50" 0.6 "60" 0.7 "70" 0.8 "80" 0.9 "90" 1 "100") ///
	ylabel(0 "0" 0.1 "10" 0.2 "20" 0.3 "30" 0.4 "40" 0.5 "50" 0.6 "60" 0.7 "70" 0.8 "80" 0.9 "90" 1 "100") ///
	legend(pos(3) col(1) label(1 "Unadjusted") label(2 "Adjusted" "via re-ranking"))
*/

// -------------------------------------------------------------------------- //
// Correction method 2: Constraining
// -------------------------------------------------------------------------- //

// Slope parameter at the juncture for tax data
levelsof q_tax if p == 90000, local(q_tax)

// Slope parameter at the juncture for tax data
levelsof q_svy if p == 90000, local(q_svy)

// Define the line (AO) of maximum equality
generate p_ao = 0.9*(l_resc/`bottomshare_tax') if (p <= 90000)

// Define the line (AB) of maximum inequality according to the tax
generate p_ab = 0.9 + (l_resc - `bottomshare_tax')/`q_tax' if (p <= 90000)

// Define the line (AC) of maximum inequality according to the tax
generate p_ac = 0.9 + (l_resc - `bottomshare_tax')/`q_svy' if (p <= 90000)

// Show the geometry of the problem
generate O = 0 if p == 0
generate A = `bottomshare_tax' if p == 90000
generate B = 0 in 1
generate C = 0 in 1

/*
gr tw (line l_resc p_std) (line l_resc p_ao) (line l_resc p_ab) (line l_resc p_ac) ///
	(scatter O p_std) (scatter A p_std) (scatter B p_ab) (scatter C p_ac), ///
	aspectratio(1) xlabel(0(0.1)1, grid) ylabel(0(0.1)1) ///
	xtitle("cumulative population share (%)") ytitle("cumulative income share") ///
	xlabel(0 "0" 0.1 "10" 0.2 "20" 0.3 "30" 0.4 "40" 0.5 "50" 0.6 "60" 0.7 "70" 0.8 "80" 0.9 "90" 1 "100") ///
	ylabel(0 "0" 0.1 "10" 0.2 "20" 0.3 "30" 0.4 "40" 0.5 "50" 0.6 "60" 0.7 "70" 0.8 "80" 0.9 "90" 1 "100") ///
	legend(pos(3) col(1) label(1 "Unadjusted" "Lorenz curve") label(2 "Minimum inequality") ///
		label(3 "Maximum" "inequality (tax)") label(4 "Maximum" "inequality (survey)"))
*/

// Constrain Lorenz curve at the bottom
generate p_cns = p_ao + (p_ab - p_ao)*(p_std - p_ao)/(p_ac - p_ao) if p < 90000
replace p_cns = p_std if p >= 90000

/*
gr tw (line l_resc p_std) (line l_resc p_cns) (line l_resc p_ao) (line l_resc p_ab) (line l_resc p_ac) ///
	(scatter O p_std) (scatter A p_std) (scatter B p_ab) (scatter C p_ac), ///
	aspectratio(1) xlabel(0(0.1)1, grid) ylabel(0(0.1)1) scale(0.8) ///
	xtitle("cumulative population share (%)") ytitle("cumulative income share") ///
	xlabel(0 "0" 0.1 "10" 0.2 "20" 0.3 "30" 0.4 "40" 0.5 "50" 0.6 "60" 0.7 "70" 0.8 "80" 0.9 "90" 1 "100") ///
	ylabel(0 "0" 0.1 "10" 0.2 "20" 0.3 "30" 0.4 "40" 0.5 "50" 0.6 "60" 0.7 "70" 0.8 "80" 0.9 "90" 1 "100") ///
	legend(pos(3) col(1) label(1 "Unadjusted" "Lorenz curve") label(2 "Constrained" "Lorenz curve") ///
		label(3 "Minimum inequality") label(4 "Maximum" "inequality (tax)") label(5 "Maximum" "inequality (survey)"))
*/

// Notice the differences with the re-ranking approach for the Lorenz curve and the quantiles
generate n_cns = cond(_n < _N, p_cns[_n + 1], 1) - p_cns
generate q_cns = (cond(_n < _N, l_resc[_n + 1], 1) - l_resc)/n_cns

/*
gr tw line q_resc q_sort q_cns p_std if p <= 98000, xtitle("population percentile (%)") ytitle("(normalized) quantile") ///
	xlabel(0 "0" 0.1 "10" 0.2 "20" 0.3 "30" 0.4 "40" 0.5 "50" 0.6 "60" 0.7 "70" 0.8 "80" 0.9 "90" 1 "100") ///
	legend(row(1) label(1 "Unadjusted") label(2 "Adjusted via re-ranking") label(3 "Adjusted via constraining"))
*/
	
gr tw (line l_resc p_std) (line l_sort p_std) (line l_resc p_cns), aspectratio(1) xlabel(0(0.1)1, grid) ylabel(0(0.1)1) ///
	xtitle("cumulative population share (%)") ytitle("cumulative income share") ///
	xlabel(0 "0" 0.1 "10" 0.2 "20" 0.3 "30" 0.4 "40" 0.5 "50" 0.6 "60" 0.7 "70" 0.8 "80" 0.9 "90" 1 "100") ///
	ylabel(0 "0" 0.1 "10" 0.2 "20" 0.3 "30" 0.4 "40" 0.5 "50" 0.6 "60" 0.7 "70" 0.8 "80" 0.9 "90" 1 "100") ///
	legend(pos(3) col(1) label(1 "Unadjusted") label(2 "Adjusted" "via re-ranking") label(3 "Adjusted" "via constraining"))

